import { AxiosInstance, AxiosResponse } from "axios";
import {
  MemberViewModel,
  CreateMemberViewModel,
  UpdateMemberViewModel,
} from "../../../viewmodels/admin/club/member/member.models";
import { RequestDefinition, RequestData } from "../../requeststore";

abstract class IMemberRequests {
  static getAllMembers: RequestDefinition<void, void, MemberViewModel[]>;
  static getMemberById: RequestDefinition<{ id: number }, void, MemberViewModel>;
  static createMember: RequestDefinition<void, CreateMemberViewModel, number>;
  static updateMember: RequestDefinition<{ id: number }, UpdateMemberViewModel, void>;
  static deleteMember: RequestDefinition<{ id: number }, void, void>;
}

export default abstract class MemberRequests extends IMemberRequests {
  static async getAllMembers(http: AxiosInstance): Promise<AxiosResponse<MemberViewModel[], any>> {
    return await http.get("/admin/member");
  }
  static async getMemberById(http: AxiosInstance, p: RequestData<{ id: number }, void>) {
    return await http.get(`/admin/member/${p.params.id}`);
  }
  static async createMember(http: AxiosInstance, p: RequestData<void, CreateMemberViewModel>) {
    return await http.post("/admin/member", p.body);
  }
  static async updateMember(http: AxiosInstance, p: RequestData<{ id: number }, UpdateMemberViewModel>) {
    return await http.post(`/admin/member/${p.params.id}`, p.body);
  }
  static async deleteMember(http: AxiosInstance, p: RequestData<{ id: number }, void>) {
    return await http.post(`/admin/member/${p.params.id}`);
  }
}
